
/*
 * Copyright (C) 2011 Mail.RU
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <tarantool/tnt.h>

#include <nb_queue.h>
#include <nb_stat.h>
#include <nb_func.h>
#include <nb_test.h>
#include <nb_opt.h>
#include <nb.h>
#include <nb_plot.h>

static int
nb_plot_data(struct nb *bench, struct nb_test *test)
{
	char file[512];
	snprintf(file, sizeof(file), "%s/bench-%s.data",
		bench->opt->plot, test->func->name);

	FILE *f = fopen(file, "w");
	if (f == NULL)
		return -1;

	char data[8096];
	int pos = 0;
	struct nb_test_buf *b;
	STAILQ_FOREACH(b, &test->list, next) {
		pos  = snprintf(data, sizeof(data), "%d\t", b->buf);
		pos += snprintf(data + pos,
			sizeof(data) - pos, "%g\t", b->stat.rps);
		pos += snprintf(data + pos,
			sizeof(data) - pos, "%g\t", b->stat_es.rps);
		fputs(data, f);
		fputc('\n', f);
	}
	fclose(f);
	return 0;
}

static int
nb_plot_cfg(struct nb *bench, struct nb_test *test)
{
	char file[512];
	snprintf(file, sizeof(file), "%s/bench-%s.plot",
		bench->opt->plot, test->func->name);

	char *head =
		"set terminal png nocrop enhanced size 1024,600 fon Arial 7\n"
		"set output '%s'\n"
		"set ytics border in scale 1,0.5 mirror norotate  offset character 0, 0, 0\n"
		"set xtics (%s)\n"
		"set title \"'%s' benchmark (reqs: %d, reps: %d, threads: %d)\"\n"
		"set xlabel \"Buffers\"\n"
		"set ylabel \"Requests\"\n";

	FILE *f = fopen(file, "w");
	if (f == NULL)
		return -1;

	fputs("# autogenerated by nb.\n\n", f);
	snprintf(file, sizeof(file), "bench-%s.png",
		test->func->name);
	fprintf(f, head, file,
		nb_test_buf_list(test),
		test->func->name,
		bench->opt->count,
		bench->opt->rep,
		bench->opt->threads);

	char *head_xrange =
		"set xrange [%d : %d] noreverse nowriteback\n\n";

	int bmin = nb_test_buf_min(test),
	    bmax = nb_test_buf_max(test);
	if (bmin == bmax) {
		bmin -= 10;
		bmax += 10;
	}

	fprintf(f, head_xrange, bmin, bmax);
	snprintf(file, sizeof(file), "bench-%s.data", test->func->name);

	char plotfmt[1024];
	snprintf(plotfmt, sizeof(plotfmt),
		"plot '%s' using 1:2 with lines t \"%s (%.2f sum)\", \\\n"
		"'%s' using 1:2:2 with labels notitle, \\\n"
		"'%s' using 1:3 with lines t \"%s (ex. threads sync) (%.2f sum)\", \\\n"
		"'%s' using 1:3:3 with labels notitle\n",
		file, test->func->name, test->integral, file,
		file, test->func->name, test->integral_es, file);

	fputs(plotfmt, f);
	fputs("\n# vim: syntax=gnuplot", f);
	fclose(f);
	return 0;
}

int
nb_plot_gen(struct nb *bench)
{
	if (bench->opt->plot) {
		struct stat st;
		if (stat(bench->opt->plot, &st) == -1) {
			if (mkdir(bench->opt->plot, 0755) == -1)
				return -1;
		}
	}
	struct nb_test *t;
	STAILQ_FOREACH(t, &bench->tests.list, next) {
		if (nb_plot_data(bench, t) == -1)
			return -1;
		if (nb_plot_cfg(bench, t) == -1)
			return -1;
	}
	return 0;
}
